@@ -3,7 +3,21 @@ import asyncio
 import os
 import shutil
 from types import TracebackType
-from typing import Any, AsyncGenerator, Callable, Coroutine, Dict, Generator, IO, Iterable, Optional, Tuple, Type, Union, List
+from typing import (
+    Any,
+    AsyncGenerator,
+    Callable,
+    Coroutine,
+    Dict,
+    Generator,
+    IO,
+    Iterable,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    List,
+)
 from aiofiles.threadpool.binary import AsyncBufferedIOBase
 
 import aiohttp
@@ -59,40 +73,48 @@ class Client(object):
             download/upload functions. Default is 65536 bytes (65K)
     """
 
-    ROOT = '/'
+    ROOT = "/"
 
     # HTTP headers for different actions
     DEFAULT_HTTP_HEADER = {
-        'list': ["Accept: */*", "Depth: 1"],
-        'free': ["Accept: */*", "Depth: 0", "Content-Type: text/xml"],
-        'copy': ["Accept: */*"],
-        'move': ["Accept: */*"],
-        'mkdir': ["Accept: */*", "Connection: Keep-Alive"],
-        'clean': ["Accept: */*", "Connection: Keep-Alive"],
-        'check': ["Accept: */*"],
-        'info': ["Accept: */*", "Depth: 1"],
-        'get_property': ["Accept: */*", "Depth: 1", "Content-Type: application/x-www-form-urlencoded"],
-        'set_property': ["Accept: */*", "Depth: 1", "Content-Type: application/x-www-form-urlencoded"]
+        "list": ["Accept: */*", "Depth: 1"],
+        "free": ["Accept: */*", "Depth: 0", "Content-Type: text/xml"],
+        "copy": ["Accept: */*"],
+        "move": ["Accept: */*"],
+        "mkdir": ["Accept: */*", "Connection: Keep-Alive"],
+        "clean": ["Accept: */*", "Connection: Keep-Alive"],
+        "check": ["Accept: */*"],
+        "info": ["Accept: */*", "Depth: 1"],
+        "get_property": [
+            "Accept: */*",
+            "Depth: 1",
+            "Content-Type: application/x-www-form-urlencoded",
+        ],
+        "set_property": [
+            "Accept: */*",
+            "Depth: 1",
+            "Content-Type: application/x-www-form-urlencoded",
+        ],
     }
 
     # mapping of actions to WebDAV methods
     DEFAULT_REQUESTS = {
-        'options': 'OPTIONS',
-        'download': "GET",
-        'upload': "PUT",
-        'copy': "COPY",
-        'move': "MOVE",
-        'mkdir': "MKCOL",
-        'clean': "DELETE",
-        'check': "HEAD",
-        'list': "PROPFIND",
-        'free': "PROPFIND",
-        'info': "PROPFIND",
-        'publish': "PROPPATCH",
-        'unpublish': "PROPPATCH",
-        'published': "PROPPATCH",
-        'get_property': "PROPFIND",
-        'set_property': "PROPPATCH"
+        "options": "OPTIONS",
+        "download": "GET",
+        "upload": "PUT",
+        "copy": "COPY",
+        "move": "MOVE",
+        "mkdir": "MKCOL",
+        "clean": "DELETE",
+        "check": "HEAD",
+        "list": "PROPFIND",
+        "free": "PROPFIND",
+        "info": "PROPFIND",
+        "publish": "PROPPATCH",
+        "unpublish": "PROPPATCH",
+        "published": "PROPPATCH",
+        "get_property": "PROPFIND",
+        "set_property": "PROPPATCH",
     }
 
     def __init__(
@@ -109,32 +131,42 @@ class Client(object):
         proxy_password: Optional[str] = None,
         insecure: Optional[bool] = False,
         loop: Optional[asyncio.AbstractEventLoop] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> None:
         self._hostname = hostname.rstrip(Urn.separate)
         self._token = token
-        self._root = (Urn(root).quote() if root else '').rstrip(Urn.separate)
+        self._root = (Urn(root).quote() if root else "").rstrip(Urn.separate)
         self._chunk_size = chunk_size if chunk_size else 65536
         self._proxy = proxy
-        self._proxy_auth = aiohttp.BasicAuth(proxy_user, proxy_password) if (
-            proxy_user and proxy_password) else None
+        self._proxy_auth = (
+            aiohttp.BasicAuth(proxy_user, proxy_password)
+            if (proxy_user and proxy_password)
+            else None
+        )
         self._insecure = insecure
-        self.session = aiohttp.ClientSession(
+        self.session = (
+            kwargs.get("session", None) or
+            aiohttp.ClientSession(
             loop=loop,
-            timeout=aiohttp.ClientTimeout(total=timeout) if timeout else DEFAULT_TIMEOUT,
-            auth=aiohttp.BasicAuth(login, password) if (
-                login and password) else None,
-            connector=kwargs.get('connector', None)
+            timeout=aiohttp.ClientTimeout(total=timeout)
+            if timeout
+            else DEFAULT_TIMEOUT,
+            auth=aiohttp.BasicAuth(login, password) if (login and password) else None,
+            connector=kwargs.get("connector", None),
+            )
         )
 
     def _get_headers(
-        self,
-        action: str,
-        headers_ext: Optional[Dict[str, str]] = None
+        self, action: str, headers_ext: Optional[Dict[str, str]] = None
     ) -> Dict[str, str]:
         headers = None
         if action in Client.DEFAULT_HTTP_HEADER:
-            headers = dict([map(lambda s: s.strip(), x.split(':', 1)) for x in Client.DEFAULT_HTTP_HEADER[action]])
+            headers = dict(
+                [
+                    map(lambda s: s.strip(), x.split(":", 1))
+                    for x in Client.DEFAULT_HTTP_HEADER[action]
+                ]
+            )
         else:
             headers = dict()
 
@@ -143,31 +175,28 @@ class Client(object):
                 headers[k] = v
 
         if self._token:
-            headers['Authorization'] = f'Bearer {self._token}'
+            headers["Authorization"] = f"Bearer {self._token}"
 
         return headers
 
-    def _get_url(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> str:
+    def _get_url(self, path: Union[str, "os.PathLike[str]"]) -> str:
         return f"{self._hostname}{self._root}{path}"
 
-    def get_full_path(
-        self,
-        urn: Urn
-    ) -> str:
+    def get_full_path(self, urn: Urn) -> str:
         return f"{self._root}{urn.path()}"
 
-    async def _execute_request(self,
-                               action: str,
-                               path: Union[str, "os.PathLike[str]"],
-                               data: Optional[Any] = None,
-                               headers_ext: Optional[Dict[str, str]] = None
-                               ) -> aiohttp.ClientResponse:
+    async def _execute_request(
+        self,
+        action: str,
+        path: Union[str, "os.PathLike[str]"],
+        data: Optional[Any] = None,
+        headers_ext: Optional[Dict[str, str]] = None,
+    ) -> aiohttp.ClientResponse:
         try:
             if self.session.auth:
-                await self.session.get(url=self._hostname)  # (Re)Authenticates against the proxy
+                await self.session.get(
+                    url=self._hostname
+                )  # (Re)Authenticates against the proxy
 
             response = await self.session.request(
                 method=Client.DEFAULT_REQUESTS[action],
@@ -177,7 +206,7 @@ class Client(object):
                 proxy=self._proxy,
                 proxy_auth=self._proxy_auth,
                 # chunked = self._chunk_size,
-                ssl=False if self._insecure else None
+                ssl=False if self._insecure else None,
             )
 
             if response.status == 507:
@@ -187,7 +216,11 @@ class Client(object):
             if response.status == 405:
                 raise MethodNotSupported(name=action, server=self._hostname)
             if response.status >= 400:
-                raise ResponseErrorCode(url=self._get_url(path), code=response.status, message=response.content)
+                raise ResponseErrorCode(
+                    url=self._get_url(path),
+                    code=response.status,
+                    message=response.content,
+                )
 
             return response
         except aiohttp.ClientConnectionError:
@@ -225,7 +258,7 @@ class Client(object):
     async def list(
         self,
         path: Optional[Union[str, "os.PathLike[str]"]] = ROOT,
-        get_info: Optional[bool] = False
+        get_info: Optional[bool] = False,
     ) -> Union[List[str], List[Dict[str, str]]]:
         """
         Returns list of nested files and directories for remote WebDAV directory by path.
@@ -252,19 +285,31 @@ class Client(object):
                 `path`: path of resource.
         """
         directory_urn = Urn(path, directory=True)
-        if directory_urn.path() != Client.ROOT and not (await self.exists(directory_urn.path())):
+        if directory_urn.path() != Client.ROOT and not (
+            await self.exists(directory_urn.path())
+        ):
             raise RemoteResourceNotFound(directory_urn.path())
 
         path = Urn.normalize_path(self.get_full_path(directory_urn))
-        response = await self._execute_request(action='list', path=directory_urn.quote())
+        response = await self._execute_request(
+            action="list", path=directory_urn.quote()
+        )
         text = await response.text()
 
         if get_info:
             subfiles = WebDavXmlUtils.parse_get_list_info_response(text)
-            return [subfile for subfile in subfiles if Urn.compare_path(path, subfile.get('path')) is False]
+            return [
+                subfile
+                for subfile in subfiles
+                if Urn.compare_path(path, subfile.get("path")) is False
+            ]
 
         urns = WebDavXmlUtils.parse_get_list_response(text)
-        return [urn.filename() for urn in urns if Urn.compare_path(path, urn.path()) is False]
+        return [
+            urn.filename()
+            for urn in urns
+            if Urn.compare_path(path, urn.path()) is False
+        ]
 
     async def free(self) -> int:
         """
@@ -276,14 +321,11 @@ class Client(object):
         """
 
         data = WebDavXmlUtils.create_free_space_request_content()
-        response = await self._execute_request(action='free', path='', data=data)
+        response = await self._execute_request(action="free", path="", data=data)
         text = await response.text()
         return WebDavXmlUtils.parse_free_space_response(text, self._hostname)
 
-    async def exists(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> bool:
+    async def exists(self, path: Union[str, "os.PathLike[str]"]) -> bool:
         """
         Checks an existence of remote resource on WebDAV server by remote path.
         More information you can find by link http://webdav.org/specs/rfc4918.html#rfc.section.9.4
@@ -298,18 +340,15 @@ class Client(object):
 
         urn = Urn(path)
         try:
-            response = await self._execute_request(action='check', path=urn.quote())
+            response = await self._execute_request(action="check", path=urn.quote())
         except RemoteResourceNotFound:
             return False
         except ResponseErrorCode:
             return False
 
-        return (int(response.status) == 200)
+        return int(response.status) == 200
 
-    async def create_directory(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> bool:
+    async def create_directory(self, path: Union[str, "os.PathLike[str]"]) -> bool:
         """
         Makes new directory on WebDAV server.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_MKCOL
@@ -327,26 +366,25 @@ class Client(object):
             raise RemoteParentNotFound(directory_urn.path())
 
         try:
-            response = await self._execute_request(action='mkdir', path=directory_urn.quote())
+            response = await self._execute_request(
+                action="mkdir", path=directory_urn.quote()
+            )
         except MethodNotSupported:
             # Yandex WebDAV returns 405 status code when directory already exists
             return True
         return response.status in (200, 201)
 
     async def _check_remote_resource(
-        self,
-        path: Union[str, "os.PathLike[str]"],
-        urn: Urn
+        self, path: Union[str, "os.PathLike[str]"], urn: Urn
     ) -> None:
-        if not (await self.exists(urn.path())) and not (await self.exists(Urn(path, directory=True).path())):
+        if not (await self.exists(urn.path())) and not (
+            await self.exists(Urn(path, directory=True).path())
+        ):
             raise RemoteResourceNotFound(path)
 
-    async def is_directory(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> bool:
+    async def is_directory(self, path: Union[str, "os.PathLike[str]"]) -> bool:
         """
-        Checks is the remote resource directory.
+        Checks if the remote resource is a directory.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFINDL
 
         Parameters:
@@ -361,15 +399,14 @@ class Client(object):
         parent_urn = Urn(urn.parent())
         await self._check_remote_resource(path, urn)
 
-        response = await self._execute_request(action='info', path=parent_urn.quote())
+        response = await self._execute_request(action="info", path=parent_urn.quote())
         text = await response.text()
         path = self.get_full_path(urn)
-        return WebDavXmlUtils.parse_is_dir_response(content=text, path=path, hostname=self._hostname)
+        return WebDavXmlUtils.parse_is_dir_response(
+            content=text, path=path, hostname=self._hostname
+        )
 
-    async def info(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> Dict[str, str]:
+    async def info(self, path: Union[str, "os.PathLike[str]"]) -> Dict[str, str]:
         """
         Gets information about resource on WebDAV.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIND
@@ -391,15 +428,14 @@ class Client(object):
         urn = Urn(path)
         await self._check_remote_resource(path, urn)
 
-        response = await self._execute_request(action='info', path=urn.quote())
+        response = await self._execute_request(action="info", path=urn.quote())
         text = await response.text()
         path = self.get_full_path(urn)
-        return WebDavXmlUtils.parse_info_response(content=text, path=path, hostname=self._hostname)
+        return WebDavXmlUtils.parse_info_response(
+            content=text, path=path, hostname=self._hostname
+        )
 
-    async def unlink(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> None:
+    async def unlink(self, path: Union[str, "os.PathLike[str]"]) -> None:
         """
         Cleans (Deletes) a remote resource on WebDAV server. The name of method is not changed for back compatibility
         with original library.
@@ -411,12 +447,9 @@ class Client(object):
         """
 
         urn = Urn(path)
-        await self._execute_request(action='clean', path=urn.quote())
+        await self._execute_request(action="clean", path=urn.quote())
 
-    async def delete(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> None:
+    async def delete(self, path: Union[str, "os.PathLike[str]"]) -> None:
         """
         Cleans (Deletes) a remote resource on WebDAV server. The name of method is not changed for back compatibility
         with original library.
@@ -433,7 +466,7 @@ class Client(object):
         self,
         source: Union[str, "os.PathLike[str]"],
         destination: Union[str, "os.PathLike[str]"],
-        overwrite: Optional[bool] = False
+        overwrite: Optional[bool] = False,
     ) -> None:
         """
         Moves resource from one place to another on WebDAV server.
@@ -459,17 +492,19 @@ class Client(object):
             raise RemoteParentNotFound(urn_to.path())
 
         headers = {
-            'Destination': self._get_url(urn_to.quote()),
-            'Overwrite': ('T' if overwrite else 'F')
+            "Destination": self._get_url(urn_to.quote()),
+            "Overwrite": ("T" if overwrite else "F"),
         }
 
-        await self._execute_request(action='move', path=urn_from.quote(), headers_ext=headers)
+        await self._execute_request(
+            action="move", path=urn_from.quote(), headers_ext=headers
+        )
 
     async def copy(
         self,
         source: Union[str, "os.PathLike[str]"],
         destination: Union[str, "os.PathLike[str]"],
-        depth: Optional[int] = 1
+        depth: Optional[int] = 1,
     ) -> None:
         """
         Copies resource from one place to another on WebDAV server.
@@ -494,17 +529,16 @@ class Client(object):
         if not (await self.exists(urn_to.parent())):
             raise RemoteParentNotFound(urn_to.path())
 
-        headers = {
-            "Destination": self._get_url(urn_to.quote())
-        }
-        if (await self.is_directory(urn_from.path())):
+        headers = {"Destination": self._get_url(urn_to.quote())}
+        if await self.is_directory(urn_from.path()):
             headers["Depth"] = depth
 
-        await self._execute_request(action='copy', path=urn_from.quote(), headers_ext=headers)
+        await self._execute_request(
+            action="copy", path=urn_from.quote(), headers_ext=headers
+        )
 
     async def download_iter(
-        self,
-        path: Union[str, "os.PathLike[str]"]
+        self, path: Union[str, "os.PathLike[str]"]
     ) -> Generator[bytes, None, None]:
         """
         Downloads file from server and return content in generator.
@@ -527,13 +561,13 @@ class Client(object):
         """
 
         urn = Urn(path)
-        if (await self.is_directory(urn.path())):
+        if await self.is_directory(urn.path()):
             raise OptionNotValid(name="path", value=path)
 
         if not (await self.exists(urn.path())):
             raise RemoteResourceNotFound(urn.path())
 
-        response = await self._execute_request(action='download', path=urn.quote())
+        response = await self._execute_request(action="download", path=urn.quote())
         return response.content.iter_chunked(self._chunk_size)
 
     async def download_to(
@@ -541,7 +575,7 @@ class Client(object):
         path: Union[str, "os.PathLike[str]"],
         buffer: IO,
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Downloads file from server and writes it in buffer.
@@ -589,14 +623,14 @@ class Client(object):
         """
 
         urn = Urn(path)
-        if (await self.is_directory(urn.path())):
+        if await self.is_directory(urn.path()):
             raise OptionNotValid(name="path", value=path)
 
         if not (await self.exists(urn.path())):
             raise RemoteResourceNotFound(urn.path())
 
-        response = await self._execute_request('download', urn.quote())
-        total = int(response.headers['content-length'])
+        response = await self._execute_request("download", urn.quote())
+        total = int(response.headers["content-length"])
         current = 0
 
         if callable(progress):
@@ -624,7 +658,7 @@ class Client(object):
         remote_path: Union[str, "os.PathLike[str]"],
         local_path: Union[str, "os.PathLike[str]"],
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Downloads file from server and write to a file.
@@ -670,15 +704,17 @@ class Client(object):
                 ...
         """
 
-        async with aiofiles.open(local_path, 'wb') as file:
-            await self.download_to(remote_path, file, progress=progress, progress_args=progress_args)
+        async with aiofiles.open(local_path, "wb") as file:
+            await self.download_to(
+                remote_path, file, progress=progress, progress_args=progress_args
+            )
 
     async def download_directory(
         self,
         remote_path: Union[str, "os.PathLike[str]"],
         local_path: Union[str, "os.PathLike[str]"],
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Downloads directory and downloads all nested files and directories from remote server to local.
@@ -730,14 +766,19 @@ class Client(object):
                 continue
             _remote_path = f"{urn.path()}{resource_name}"
             _local_path = os.path.join(local_path, resource_name)
-            await self.download(local_path=_local_path, remote_path=_remote_path, progress=progress, progress_args=progress_args)
+            await self.download(
+                local_path=_local_path,
+                remote_path=_remote_path,
+                progress=progress,
+                progress_args=progress_args,
+            )
 
     async def download(
         self,
         remote_path: Union[str, "os.PathLike[str]"],
         local_path: Union[str, "os.PathLike[str]"],
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Download a remote resourse and put in local path.
@@ -775,18 +816,29 @@ class Client(object):
         """
 
         urn = Urn(remote_path)
-        if (await self.is_directory(urn.path())):
-            await self.download_directory(local_path=local_path, remote_path=remote_path, progress=progress, progress_args=progress_args)
+        if await self.is_directory(urn.path()):
+            await self.download_directory(
+                local_path=local_path,
+                remote_path=remote_path,
+                progress=progress,
+                progress_args=progress_args,
+            )
         else:
-            await self.download_file(local_path=local_path, remote_path=remote_path, progress=progress, progress_args=progress_args)
+            await self.download_file(
+                local_path=local_path,
+                remote_path=remote_path,
+                progress=progress,
+                progress_args=progress_args,
+            )
 
     async def upload_to(
         self,
         path: Union[str, "os.PathLike[str]"],
         buffer: Union[IO, AsyncGenerator[bytes, None]],
         buffer_size: Optional[int] = None,
+        overwrite: bool = True,
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Uploads file from buffer to remote path on WebDAV server.
@@ -800,6 +852,10 @@ class Client(object):
                 IO like object to read the data or a asynchronous generator to get buffer data.
                 In order do you select use a async generator `progress` callback cannot be called.
 
+            overwrite (``bool``)
+                If true the will be overwriten by the new data if the a file with the same path exists. 
+                (Default to true)
+
             progress (``callable``, *optional*):
                 Pass a callback function to view the file transmission progress.
                 The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
@@ -838,10 +894,14 @@ class Client(object):
         if urn.is_dir():
             raise OptionNotValid(name="path", value=path)
 
+        if not overwrite and (await self.exists(urn.path())):
+            return
+
         if not (await self.exists(urn.parent())):
             raise RemoteParentNotFound(urn.path())
 
         if callable(progress) and not asyncio.iscoroutinefunction(buffer):
+
             async def file_sender(buff: IO):
                 current = 0
 
@@ -851,8 +911,11 @@ class Client(object):
                     progress(current, buffer_size, *progress_args)
 
                 while current < buffer_size:
-                    chunk = await buffer.read(self._chunk_size) if isinstance(buffer, AsyncBufferedIOBase) \
+                    chunk = (
+                        await buffer.read(self._chunk_size)
+                        if isinstance(buffer, AsyncBufferedIOBase)
                         else buffer.read(self._chunk_size)
+                    )
                     if not chunk:
                         break
 
@@ -864,16 +927,19 @@ class Client(object):
                         progress(current, buffer_size, *progress_args)
                     yield chunk
 
-            await self._execute_request(action='upload', path=urn.quote(), data=file_sender(buffer))
+            await self._execute_request(
+                action="upload", path=urn.quote(), data=file_sender(buffer)
+            )
         else:
-            await self._execute_request(action='upload', path=urn.quote(), data=buffer)
+            await self._execute_request(action="upload", path=urn.quote(), data=buffer)
 
     async def upload_file(
         self,
         remote_path: Union[str, "os.PathLike[str]"],
         local_path: Union[str, "os.PathLike[str]"],
+        overwrite: bool = True,
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Uploads file to remote path on WebDAV server. File should be 2Gb or less.
@@ -886,6 +952,10 @@ class Client(object):
             local_path (``str``):
                 The path to local file for uploading.
 
+            overwrite (``bool``)
+                If true the will be overwriten by the new data if the a file with the same path exists. 
+                (Default to true)
+
             progress (``callable``, *optional*):
                 Pass a callback function to view the file transmission progress.
                 The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
@@ -919,17 +989,24 @@ class Client(object):
                 ...
         """
 
-        async with aiofiles.open(local_path, 'rb') as file:
+        async with aiofiles.open(local_path, "rb") as file:
             size = os.path.getsize(local_path)
-            await self.upload_to(path=remote_path, buffer=file, buffer_size=size,
-                                 progress=progress, progress_args=progress_args)
+            await self.upload_to(
+                path=remote_path,
+                buffer=file,
+                buffer_size=size,
+                overwrite=overwrite,
+                progress=progress,
+                progress_args=progress_args,
+            )
 
     async def upload_directory(
         self,
         remote_path: Union[str, "os.PathLike[str]"],
         local_path: Union[str, "os.PathLike[str]"],
+        overwrite: bool = True,
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Uploads directory to remote path on WebDAV server. In case directory is exist
@@ -946,6 +1023,10 @@ class Client(object):
             local_path (``str``):
                 The path to local directory for uploading.
 
+            overwrite (``bool``)
+                If true the will be overwriten by the new data if the directory with the same path exists. 
+                (Default to true)
+
             progress (``callable``, *optional*):
                 Pass a callback function to view the file transmission progress.
                 The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
@@ -979,21 +1060,29 @@ class Client(object):
             raise LocalResourceNotFound(local_path)
 
         if (await self.exists(urn.path())):
-            await self.unlink(urn.path())
+            if overwrite:
+                await self.unlink(urn.path())
+            else:
+                return
 
         await self.create_directory(remote_path)
 
         for resource_name in os.listdir(local_path):
-            _remote_path = f"{urn.path()}{resource_name}".replace('\\', '')
+            _remote_path = f"{urn.path()}{resource_name}".replace("\\", "")
             _local_path = os.path.join(local_path, resource_name)
-            await self.upload(local_path=_local_path, remote_path=_remote_path, progress=progress, progress_args=progress_args)
+            await self.upload(
+                local_path=_local_path,
+                remote_path=_remote_path,
+                progress=progress,
+                progress_args=progress_args,
+            )
 
     async def upload(
         self,
         remote_path: Union[str, "os.PathLike[str]"],
         local_path: Union[str, "os.PathLike[str]"],
         progress: Optional[Callable[[int, int, Tuple], None]] = None,
-        progress_args: Optional[Tuple] = ()
+        progress_args: Optional[Tuple] = (),
     ) -> None:
         """
         Uploads resource to remote path on WebDAV server.
@@ -1032,14 +1121,22 @@ class Client(object):
         """
 
         if os.path.isdir(local_path):
-            self.upload_directory(local_path=local_path, remote_path=remote_path, progress=progress, progress_args=progress_args)
+            self.upload_directory(
+                local_path=local_path,
+                remote_path=remote_path,
+                progress=progress,
+                progress_args=progress_args,
+            )
         else:
-            self.upload_file(local_path=local_path, remote_path=remote_path, progress=progress, progress_args=progress_args)
+            self.upload_file(
+                local_path=local_path,
+                remote_path=remote_path,
+                progress=progress,
+                progress_args=progress_args,
+            )
 
     async def get_property(
-        self,
-        path: Union[str, "os.PathLike[str]"],
-        option: Dict[str, str]
+        self, path: Union[str, "os.PathLike[str]"], option: Dict[str, str]
     ) -> Union[str, None]:
         """
         Gets metadata property of remote resource on WebDAV server.
@@ -1062,14 +1159,14 @@ class Client(object):
             raise RemoteResourceNotFound(urn.path())
 
         data = WebDavXmlUtils.create_get_property_request_content(option)
-        response = await self._execute_request(action='get_property', path=urn.quote(), data=data)
+        response = await self._execute_request(
+            action="get_property", path=urn.quote(), data=data
+        )
         text = await response.text()
-        return WebDavXmlUtils.parse_get_property_response(text, option['name'])
+        return WebDavXmlUtils.parse_get_property_response(text, option["name"])
 
     async def set_property(
-        self,
-        path: Union[str, "os.PathLike[str]"],
-        option: Dict[str, str]
+        self, path: Union[str, "os.PathLike[str]"], option: Dict[str, str]
     ) -> None:
         """
         Sets metadata property of remote resource on WebDAV server.
@@ -1090,7 +1187,7 @@ class Client(object):
             raise RemoteResourceNotFound(urn.path())
 
         data = WebDavXmlUtils.create_set_property_batch_request_content(option)
-        await self._execute_request(action='set_property', path=urn.quote(), data=data)
+        await self._execute_request(action="set_property", path=urn.quote(), data=data)
 
     async def close(self):
         """
@@ -1100,10 +1197,7 @@ class Client(object):
 
         await self.session.close()
 
-    def resource(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> "Resource":
+    def resource(self, path: Union[str, "os.PathLike[str]"]) -> "Resource":
         """
         Get associated resource from path
 
@@ -1124,11 +1218,7 @@ class Resource(object):
     Remote resource.
     """
 
-    def __init__(
-        self,
-        client: Client,
-        urn: Urn
-    ) -> None:
+    def __init__(self, client: Client, urn: Urn) -> None:
         self.client = client
         self.urn = urn
 
@@ -1144,10 +1234,7 @@ class Resource(object):
         """
         return await self.client.is_directory(self.urn.path())
 
-    async def rename(
-        self,
-        name: str
-    ) -> None:
+    async def rename(self, name: str) -> None:
         """
         Rename the resource.
 
@@ -1163,10 +1250,7 @@ class Resource(object):
         await self.client.move(source=old_path, destination=new_path)
         self.urn = Urn(new_path)
 
-    async def move(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> None:
+    async def move(self, path: Union[str, "os.PathLike[str]"]) -> None:
         """
         Move the resource to new path.
 
@@ -1178,10 +1262,7 @@ class Resource(object):
         await self.client.move(source=self.urn.path(), destination=new_urn.path())
         self.urn = new_urn
 
-    async def copy(
-        self,
-        path: Union[str, "os.PathLike[str]"]
-    ) -> "Resource":
+    async def copy(self, path: Union[str, "os.PathLike[str]"]) -> "Resource":
         """
         Copy the resource to a another path.
 
@@ -1196,10 +1277,7 @@ class Resource(object):
         await self.client.copy(source=self.urn.path(), destination=path)
         return Resource(self.client, urn)
 
-    async def info(
-        self,
-        filter: Optional[Iterable[str]] = None
-    ) -> Dict[str, str]:
+    async def info(self, filter: Optional[Iterable[str]] = None) -> Dict[str, str]:
         """
         Get a dictionary with resource information.
 
